@@ -3,16 +3,16 @@
  *
  * Copyright (c) 2002-2004 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors:
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: GenModelEditor.java,v 1.10 2004/07/16 16:27:26 marcelop Exp $
+ * $Id: GenModelEditor.java,v 1.15 2005/06/12 13:23:31 emerks Exp $
  */
 package org.eclipse.emf.codegen.ecore.genmodel.presentation;
 
@@ -23,7 +23,6 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.EventObject;
 import java.util.Iterator;
-import java.util.List;
 import java.util.Map;
 
 import org.eclipse.core.resources.IFile;
@@ -86,7 +85,6 @@ import org.eclipse.ui.views.properties.PropertySheetPage;
 
 import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
 import org.eclipse.emf.codegen.ecore.genmodel.provider.GenModelEditPlugin;
-import org.eclipse.emf.codegen.ecore.genmodel.provider.GenModelItemProviderAdapterFactory;
 import org.eclipse.emf.common.command.BasicCommandStack;
 import org.eclipse.emf.common.command.Command;
 import org.eclipse.emf.common.command.CommandStack;
@@ -94,7 +92,7 @@ import org.eclipse.emf.common.command.CommandStackListener;
 import org.eclipse.emf.common.notify.AdapterFactory;
 import org.eclipse.emf.common.ui.viewer.IViewerProvider;
 import org.eclipse.emf.common.util.URI;
-import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
+import org.eclipse.emf.ecore.plugin.EcorePlugin;
 import org.eclipse.emf.ecore.resource.Resource;
 import org.eclipse.emf.ecore.resource.ResourceSet;
 import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
@@ -102,7 +100,6 @@ import org.eclipse.emf.edit.domain.EditingDomain;
 import org.eclipse.emf.edit.domain.IEditingDomainProvider;
 import org.eclipse.emf.edit.provider.AdapterFactoryItemDelegator;
 import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
-import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
 import org.eclipse.emf.edit.ui.action.EditingDomainActionBarContributor;
 import org.eclipse.emf.edit.ui.celleditor.AdapterFactoryTreeEditor;
 import org.eclipse.emf.edit.ui.dnd.EditingDomainViewerDropAdapter;
@@ -245,7 +242,6 @@ public class GenModelEditor
     {
       public void resourceChanged(IResourceChangeEvent event)
       {
-        ResourceSet resourceSet = editingDomain.getResourceSet();
         // Only listening to these.
         // if (event.getType() == IResourceDelta.POST_CHANGE)
         {
@@ -296,21 +292,27 @@ public class GenModelEditor
             ResourceDeltaVisitor visitor = new ResourceDeltaVisitor();
             delta.accept(visitor);
 
-            removedResources.addAll(visitor.getRemovedResources());
-            if (!visitor.getRemovedResources().isEmpty() && !isDirty())
+            if (!visitor.getRemovedResources().isEmpty())
             {
-              getSite().getShell().getDisplay().asyncExec
-                (new Runnable()
-                 {
-                   public void run()
+              removedResources.addAll(visitor.getRemovedResources());
+              if (!isDirty())
+              {
+                getSite().getShell().getDisplay().asyncExec
+                  (new Runnable()
                    {
-                     getSite().getPage().closeEditor(GenModelEditor.this, false);
-                     GenModelEditor.this.dispose();
-                   }
-                 });
+                     public void run()
+                     {
+                       getSite().getPage().closeEditor(GenModelEditor.this, false);
+                       GenModelEditor.this.dispose();
+                     }
+                   });
+              }
             }
 
-            changedResources.addAll(visitor.getChangedResources());
+            if (!visitor.getChangedResources().isEmpty())
+            {
+              changedResources.addAll(visitor.getChangedResources());
+            }
           }
           catch (CoreException exception)
           {
@@ -441,14 +443,7 @@ public class GenModelEditor
   {
     super();
 
-    // Create an adapter factory that yields item providers.
-    //
-    List factories = new ArrayList();
-    factories.add(new GenModelItemProviderAdapterFactory());
-    factories.add(new ResourceItemProviderAdapterFactory());
-    factories.add(new EcoreItemProviderAdapterFactory());
-
-    adapterFactory = new ComposedAdapterFactory(factories);
+    adapterFactory = new ComposedAdapterFactory(ComposedAdapterFactory.Descriptor.Registry.INSTANCE);
 
     // Create the command stack that will notify this editor as commands are executed.
     //
@@ -684,6 +679,8 @@ public class GenModelEditor
     //
     IFileEditorInput modelFile = (IFileEditorInput)getEditorInput();
 
+    editingDomain.getResourceSet().getURIConverter().getURIMap().putAll(EcorePlugin.computePlatformURIMap());
+    
     try
     {
       // Load the resource through the editing domain.
@@ -716,8 +713,7 @@ public class GenModelEditor
       new AdapterFactoryTreeEditor(treeViewer.getTree(), adapterFactory);
 
       createContextMenuFor(treeViewer);
-      int pageIndex = addPage(tree);
-      // setPageText(pageIndex, "Selection");
+      addPage(tree);
     }
     setActivePage(0);
 
