@@ -3,16 +3,16 @@
  *
  * Copyright (c) 2002-2004 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors: 
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: JMerger.java,v 1.7 2004/08/10 14:00:48 emerks Exp $
+ * $Id: JMerger.java,v 1.12 2005/06/29 21:13:11 davidms Exp $
  */
 package org.eclipse.emf.codegen.jmerge;
 
@@ -36,6 +36,8 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import org.eclipse.core.runtime.IPlatformRunnable;
+import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.jdt.core.jdom.DOMFactory;
 import org.eclipse.jdt.core.jdom.IDOMCompilationUnit;
 import org.eclipse.jdt.core.jdom.IDOMField;
@@ -585,8 +587,6 @@ public class JMerger implements IPlatformRunnable
       this.string = string;
       this.stringBuffer = new StringBuffer();
 
-      int length = string.length();
-      List matches = new ArrayList();
       for (int start = 0, end = string.length(); start < end; )
       {
         Matcher matcher = pattern.matcher(string.subSequence(start, end));
@@ -735,15 +735,27 @@ public class JMerger implements IPlatformRunnable
                 String oldStringValue = (String)sourceGetMethod.invoke(targetNode, noArguments);
                 Matcher sourceMatcher = sourceTransfer.matcher(stringValue);
                 Matcher targetMatcher = sourceTransfer.matcher(oldStringValue);
-                if (sourceMatcher.find() &&
-                      targetMatcher.find() &&
-                      sourceMatcher.groupCount() >= 1 &&
-                      targetMatcher.groupCount() >= 1)
+                if (sourceMatcher.groupCount() >= 1 && targetMatcher.groupCount() >= 1)
                 {
-                  stringValue =
-                    stringValue.substring(0, sourceMatcher.start(1)) + 
-                      targetMatcher.group(1) +
-                      stringValue.substring(sourceMatcher.end(1));
+                  StringBuffer result = new StringBuffer();
+                  int index = 0;
+                  while (sourceMatcher.find() && targetMatcher.find())
+                  {
+                    result.append(stringValue.substring(index, sourceMatcher.start(1)));
+                    result.append(targetMatcher.group(1));
+                    index =  sourceMatcher.end(1);
+                  }
+                  // There must be at least one match.
+                  //
+                  if (result.length() == 0)
+                  {
+                    stringValue = null;
+                  }
+                  else
+                  {
+                    result.append(stringValue.substring(index));
+                    stringValue = result.toString();
+                  }
                 }
                 else
                 {
@@ -772,6 +784,17 @@ public class JMerger implements IPlatformRunnable
                 }
               }
 
+              // The block pattern needs to prevent merging of the return type, to allow changing from the modeled data
+              // type (Bugzilla 102209). 
+              //
+              if (sourceGetMethod.getName().equals("getReturnType") &&
+                  jControlModel.getBlockPattern() != null && 
+                  ((IDOMMethod)targetNode).getComment() != null &&
+                  jControlModel.getBlockPattern().matcher(((IDOMMethod)targetNode).getComment()).find())
+              {
+                continue;
+              }
+
               targetPutMethod.invoke(targetNode, new Object [] { value });
               if (targetPutMethod.getName().equals("setBody") && sourceNode instanceof IDOMMethod)
               {
@@ -1191,34 +1214,10 @@ public class JMerger implements IPlatformRunnable
     {
       // Three arguments are expected: the .xml jControlModel URI, the source java URI, and the target java URI.
       //
-      String[] arguments = (String[])object;
-
-      // Create the options model.
-      //
-      jControlModel = new JControlModel(arguments[0]);
-
-      // Create the source and target JDOMs.
-      //
-      sourceCompilationUnit = createCompilationUnitForURI(arguments[1]);
-      targetCompilationUnit = createCompilationUnitForURI(arguments[2]);
-
-      // Create a pattern dictionary for each.
-      //
-      sourcePatternDictionary = new JPatternDictionary(sourceCompilationUnit, jControlModel);
-      targetPatternDictionary = new JPatternDictionary(targetCompilationUnit, jControlModel);
-
-      // Pull changes from the source and then push unmapped source into the target.
-      //
-      pullTargetCompilationUnit();
-      if (!isBlocked)
-      {
-        pushSourceCompilationUnit();
-        sweepTargetCompilationUnit();
-        sortTargetCompilationUnit();
-      }
+      String contents = execute(new NullProgressMonitor(), (String[])object);
 
       System.out.println("**********************************************");
-      System.out.println(targetCompilationUnit.getContents());
+      System.out.println(contents);
 
       return new Integer(0);
     }
@@ -1228,4 +1227,38 @@ public class JMerger implements IPlatformRunnable
       return new Integer(1);
     }
   }
+  
+  /**
+   * Utilitiy for headless operations.
+   * 
+   * @param mergeXML
+   * @param sourceURI
+   * @param targetURI
+   * @return the merged content
+   * @since 2.1.0
+   */
+  public String execute(IProgressMonitor progressMonitor, String[] arguments)
+  {
+    String mergeXML = arguments[0];
+    String sourceURI = arguments[1];
+    String targetURI = arguments[2];
+    
+    // Create the options model.
+    //
+    jControlModel = new JControlModel(mergeXML);
+
+    // Create the source and target JDOMs.
+    //
+    sourceCompilationUnit = createCompilationUnitForURI(sourceURI);
+    targetCompilationUnit = createCompilationUnitForURI(targetURI);
+
+    // Create a pattern dictionary for each.
+    //
+    sourcePatternDictionary = new JPatternDictionary(sourceCompilationUnit, jControlModel);
+    targetPatternDictionary = new JPatternDictionary(targetCompilationUnit, jControlModel);
+    
+    merge();
+    
+    return targetCompilationUnit.getContents();
+  }
 }
