@@ -1,18 +1,18 @@
 /**
  * <copyright>
  *
- * Copyright (c) 2002-2004 IBM Corporation and others.
+ * Copyright (c) 2002-2005 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors: 
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: EStructuralFeatureImpl.java,v 1.5 2004/06/18 09:52:06 emerks Exp $
+ * $Id: EStructuralFeatureImpl.java,v 1.12 2005/06/12 13:29:22 emerks Exp $
  */
 package org.eclipse.emf.ecore.impl;
 
@@ -78,9 +78,15 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
   protected int featureID = -1;
   protected Class containerClass;
 
+  /**
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @generated
+   */
   protected EStructuralFeatureImpl()
   {
     super();
+    eFlags |= CHANGEABLE_EFLAG;
   }
 
   /**
@@ -116,7 +122,11 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
       if (factory != defaultValueFactory)
       {
         defaultValueFactory = factory;
-        defaultValue = factory.createFromString((EDataType)eType, literal);
+        EDataType eDataType = (EDataType)eType;
+        if (eDataType.isSerializable())
+        {
+          defaultValue = factory.createFromString((EDataType)eType, literal);
+        }
       }
       return defaultValue;
     }
@@ -163,7 +173,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public boolean isUnsettable()
   {
-    return unsettable;
+    return (eFlags & UNSETTABLE_EFLAG) != 0;
   }
 
   /**
@@ -173,10 +183,10 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public void setUnsettable(boolean newUnsettable)
   {
-    boolean oldUnsettable = unsettable;
-    unsettable = newUnsettable;
+    boolean oldUnsettable = (eFlags & UNSETTABLE_EFLAG) != 0;
+    if (newUnsettable) eFlags |= UNSETTABLE_EFLAG; else eFlags &= ~UNSETTABLE_EFLAG;
     if (eNotificationRequired())
-      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__UNSETTABLE, oldUnsettable, unsettable));
+      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__UNSETTABLE, oldUnsettable, newUnsettable));
   }
 
   /**
@@ -186,7 +196,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public boolean isDerived()
   {
-    return derived;
+    return (eFlags & DERIVED_EFLAG) != 0;
   }
 
   /**
@@ -196,10 +206,10 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public void setDerived(boolean newDerived)
   {
-    boolean oldDerived = derived;
-    derived = newDerived;
+    boolean oldDerived = (eFlags & DERIVED_EFLAG) != 0;
+    if (newDerived) eFlags |= DERIVED_EFLAG; else eFlags &= ~DERIVED_EFLAG;
     if (eNotificationRequired())
-      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__DERIVED, oldDerived, derived));
+      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__DERIVED, oldDerived, newDerived));
   }
 
   /**
@@ -224,14 +234,14 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
   protected static final boolean CHANGEABLE_EDEFAULT = true;
 
   /**
-   * The cached value of the '{@link #isChangeable() <em>Changeable</em>}' attribute.
+   * The flag representing the value of the '{@link #isChangeable() <em>Changeable</em>}' attribute.
    * <!-- begin-user-doc -->
    * <!-- end-user-doc -->
    * @see #isChangeable()
    * @generated
    * @ordered
    */
-  protected boolean changeable = CHANGEABLE_EDEFAULT;
+  protected static final int CHANGEABLE_EFLAG = 1 << 10;
 
   /**
    * The default value of the '{@link #isVolatile() <em>Volatile</em>}' attribute.
@@ -244,14 +254,14 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
   protected static final boolean VOLATILE_EDEFAULT = false;
 
   /**
-   * The cached value of the '{@link #isVolatile() <em>Volatile</em>}' attribute.
+   * The flag representing the value of the '{@link #isVolatile() <em>Volatile</em>}' attribute.
    * <!-- begin-user-doc -->
    * <!-- end-user-doc -->
    * @see #isVolatile()
    * @generated
    * @ordered
    */
-  protected boolean volatile_ = VOLATILE_EDEFAULT;
+  protected static final int VOLATILE_EFLAG = 1 << 11;
 
   /**
    * The default value of the '{@link #isTransient() <em>Transient</em>}' attribute.
@@ -264,14 +274,14 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
   protected static final boolean TRANSIENT_EDEFAULT = false;
 
   /**
-   * The cached value of the '{@link #isTransient() <em>Transient</em>}' attribute.
+   * The flag representing the value of the '{@link #isTransient() <em>Transient</em>}' attribute.
    * <!-- begin-user-doc -->
    * <!-- end-user-doc -->
    * @see #isTransient()
    * @generated
    * @ordered
    */
-  protected boolean transient_ = TRANSIENT_EDEFAULT;
+  protected static final int TRANSIENT_EFLAG = 1 << 12;
 
   /**
    * The default value of the '{@link #getDefaultValueLiteral() <em>Default Value Literal</em>}' attribute.
@@ -314,14 +324,14 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
   protected static final boolean UNSETTABLE_EDEFAULT = false;
 
   /**
-   * The cached value of the '{@link #isUnsettable() <em>Unsettable</em>}' attribute.
+   * The flag representing the value of the '{@link #isUnsettable() <em>Unsettable</em>}' attribute.
    * <!-- begin-user-doc -->
    * <!-- end-user-doc -->
    * @see #isUnsettable()
    * @generated
    * @ordered
    */
-  protected boolean unsettable = UNSETTABLE_EDEFAULT;
+  protected static final int UNSETTABLE_EFLAG = 1 << 13;
 
   /**
    * The default value of the '{@link #isDerived() <em>Derived</em>}' attribute.
@@ -334,14 +344,14 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
   protected static final boolean DERIVED_EDEFAULT = false;
 
   /**
-   * The cached value of the '{@link #isDerived() <em>Derived</em>}' attribute.
+   * The flag representing the value of the '{@link #isDerived() <em>Derived</em>}' attribute.
    * <!-- begin-user-doc -->
    * <!-- end-user-doc -->
    * @see #isDerived()
    * @generated
    * @ordered
    */
-  protected boolean derived = DERIVED_EDEFAULT;
+  protected static final int DERIVED_EFLAG = 1 << 14;
 
   /**
    * <!-- begin-user-doc -->
@@ -405,9 +415,9 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
       case EcorePackage.ESTRUCTURAL_FEATURE__NAME:
         return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
       case EcorePackage.ESTRUCTURAL_FEATURE__ORDERED:
-        return ordered != ORDERED_EDEFAULT;
+        return ((eFlags & ORDERED_EFLAG) != 0) != ORDERED_EDEFAULT;
       case EcorePackage.ESTRUCTURAL_FEATURE__UNIQUE:
-        return unique != UNIQUE_EDEFAULT;
+        return ((eFlags & UNIQUE_EFLAG) != 0) != UNIQUE_EDEFAULT;
       case EcorePackage.ESTRUCTURAL_FEATURE__LOWER_BOUND:
         return lowerBound != LOWER_BOUND_EDEFAULT;
       case EcorePackage.ESTRUCTURAL_FEATURE__UPPER_BOUND:
@@ -419,19 +429,19 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
       case EcorePackage.ESTRUCTURAL_FEATURE__ETYPE:
         return eType != null;
       case EcorePackage.ESTRUCTURAL_FEATURE__CHANGEABLE:
-        return changeable != CHANGEABLE_EDEFAULT;
+        return ((eFlags & CHANGEABLE_EFLAG) != 0) != CHANGEABLE_EDEFAULT;
       case EcorePackage.ESTRUCTURAL_FEATURE__VOLATILE:
-        return volatile_ != VOLATILE_EDEFAULT;
+        return ((eFlags & VOLATILE_EFLAG) != 0) != VOLATILE_EDEFAULT;
       case EcorePackage.ESTRUCTURAL_FEATURE__TRANSIENT:
-        return transient_ != TRANSIENT_EDEFAULT;
+        return ((eFlags & TRANSIENT_EFLAG) != 0) != TRANSIENT_EDEFAULT;
       case EcorePackage.ESTRUCTURAL_FEATURE__DEFAULT_VALUE_LITERAL:
         return DEFAULT_VALUE_LITERAL_EDEFAULT == null ? defaultValueLiteral != null : !DEFAULT_VALUE_LITERAL_EDEFAULT.equals(defaultValueLiteral);
       case EcorePackage.ESTRUCTURAL_FEATURE__DEFAULT_VALUE:
         return DEFAULT_VALUE_EDEFAULT == null ? getDefaultValue() != null : !DEFAULT_VALUE_EDEFAULT.equals(getDefaultValue());
       case EcorePackage.ESTRUCTURAL_FEATURE__UNSETTABLE:
-        return unsettable != UNSETTABLE_EDEFAULT;
+        return ((eFlags & UNSETTABLE_EFLAG) != 0) != UNSETTABLE_EDEFAULT;
       case EcorePackage.ESTRUCTURAL_FEATURE__DERIVED:
-        return derived != DERIVED_EDEFAULT;
+        return ((eFlags & DERIVED_EFLAG) != 0) != DERIVED_EDEFAULT;
       case EcorePackage.ESTRUCTURAL_FEATURE__ECONTAINING_CLASS:
         return getEContainingClass() != null;
     }
@@ -550,7 +560,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public boolean isTransient()
   {
-    return transient_;
+    return (eFlags & TRANSIENT_EFLAG) != 0;
   }
 
   /**
@@ -560,10 +570,10 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public void setTransient(boolean newTransient)
   {
-    boolean oldTransient = transient_;
-    transient_ = newTransient;
+    boolean oldTransient = (eFlags & TRANSIENT_EFLAG) != 0;
+    if (newTransient) eFlags |= TRANSIENT_EFLAG; else eFlags &= ~TRANSIENT_EFLAG;
     if (eNotificationRequired())
-      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__TRANSIENT, oldTransient, transient_));
+      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__TRANSIENT, oldTransient, newTransient));
   }
 
   /**
@@ -573,7 +583,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public boolean isVolatile()
   {
-    return volatile_;
+    return (eFlags & VOLATILE_EFLAG) != 0;
   }
 
   /**
@@ -583,10 +593,10 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public void setVolatile(boolean newVolatile)
   {
-    boolean oldVolatile = volatile_;
-    volatile_ = newVolatile;
+    boolean oldVolatile = (eFlags & VOLATILE_EFLAG) != 0;
+    if (newVolatile) eFlags |= VOLATILE_EFLAG; else eFlags &= ~VOLATILE_EFLAG;
     if (eNotificationRequired())
-      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__VOLATILE, oldVolatile, volatile_));
+      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__VOLATILE, oldVolatile, newVolatile));
   }
 
   /**
@@ -596,7 +606,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public boolean isChangeable()
   {
-    return changeable;
+    return (eFlags & CHANGEABLE_EFLAG) != 0;
   }
 
   /**
@@ -606,10 +616,10 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
    */
   public void setChangeable(boolean newChangeable)
   {
-    boolean oldChangeable = changeable;
-    changeable = newChangeable;
+    boolean oldChangeable = (eFlags & CHANGEABLE_EFLAG) != 0;
+    if (newChangeable) eFlags |= CHANGEABLE_EFLAG; else eFlags &= ~CHANGEABLE_EFLAG;
     if (eNotificationRequired())
-      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__CHANGEABLE, oldChangeable, changeable));
+      eNotify(new ENotificationImpl(this, Notification.SET, EcorePackage.ESTRUCTURAL_FEATURE__CHANGEABLE, oldChangeable, newChangeable));
   }
 
   /**
@@ -633,17 +643,17 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
 
     StringBuffer result = new StringBuffer(super.toString());
     result.append(" (changeable: ");
-    result.append(changeable);
+    result.append((eFlags & CHANGEABLE_EFLAG) != 0);
     result.append(", volatile: ");
-    result.append(volatile_);
+    result.append((eFlags & VOLATILE_EFLAG) != 0);
     result.append(", transient: ");
-    result.append(transient_);
+    result.append((eFlags & TRANSIENT_EFLAG) != 0);
     result.append(", defaultValueLiteral: ");
     result.append(defaultValueLiteral);
     result.append(", unsettable: ");
-    result.append(unsettable);
+    result.append((eFlags & UNSETTABLE_EFLAG) != 0);
     result.append(", derived: ");
-    result.append(derived);
+    result.append((eFlags & DERIVED_EFLAG) != 0);
     result.append(')');
     return result.toString();
   }
@@ -729,12 +739,12 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
       switch (eContainerFeatureID)
       {
         case EcorePackage.ESTRUCTURAL_FEATURE__ECONTAINING_CLASS:
-          return ((InternalEObject)eContainer).eInverseRemove(this, EcorePackage.ECLASS__ESTRUCTURAL_FEATURES, EClass.class, msgs);
+          return eContainer.eInverseRemove(this, EcorePackage.ECLASS__ESTRUCTURAL_FEATURES, EClass.class, msgs);
         default:
           return eDynamicBasicRemoveFromContainer(msgs);
       }
     }
-    return ((InternalEObject)eContainer).eInverseRemove(this, EOPPOSITE_FEATURE_BASE - eContainerFeatureID, null, msgs);
+    return eContainer.eInverseRemove(this, EOPPOSITE_FEATURE_BASE - eContainerFeatureID, null, msgs);
   }
 
   public void setContainerClass(Class containerClass)
@@ -769,11 +779,11 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
     if (settingDelegate == null)
     {
       EClass eClass  = getEContainingClass();
-      eClass.getEAllStructuralFeatures();
+      eClass.getFeatureCount();
       EReference eOpposite = getEOpposite();
       if (eOpposite != null)
       {
-        eOpposite.getEContainingClass().getEAllStructuralFeatures();
+        eOpposite.getEContainingClass().getFeatureCount();
       }
 
       EClassifier eType = getEType();
@@ -1391,7 +1401,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
     {
       this.style = style;
       this.dataClass = Object.class;
-      this.dynamicKind = EcoreEList.Dynamic.kind(feature);
+      this.dynamicKind = EcoreEList.Generic.kind(feature);
       this.feature = feature;
     }
 
@@ -1407,7 +1417,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
     {
       this.style = style;
       this.dataClass = Object.class;
-      this.dynamicKind = EcoreEList.Dynamic.kind(feature);
+      this.dynamicKind = EcoreEList.Generic.kind(feature);
       this.feature = feature;
       this.inverseFeature = inverseFeature;
     }
@@ -1656,7 +1666,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
       }
 
       EObject eContainer = owner.eContainer();
-      int featureID = owner.eClass().getEAllStructuralFeatures().indexOf(feature);
+      int featureID = owner.eClass().getFeatureID(feature);
       if (newValue != eContainer || (owner.eContainerFeatureID() != featureID && newValue != null))
       {
         if (EcoreUtil.isAncestor(owner, (EObject)newValue))
@@ -1673,7 +1683,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
         {
           notifications = 
             internalEObject.eInverseAdd
-              (owner, internalEObject.eClass().getEAllStructuralFeatures().indexOf(inverseFeature), null, notifications);
+              (owner, internalEObject.eClass().getFeatureID(inverseFeature), null, notifications);
         }
 
         notifications = owner.eBasicSetContainer(internalEObject, featureID, notifications);
@@ -1696,7 +1706,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
         {
           notifications = owner.eBasicRemoveFromContainer(notifications);
         }
-        int featureID = owner.eClass().getEAllStructuralFeatures().indexOf(feature);
+        int featureID = owner.eClass().getFeatureID(feature);
         notifications = owner.eBasicSetContainer(null, featureID, notifications);
         if (notifications != null) notifications.dispatch();
       }
@@ -1709,7 +1719,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
 
     public boolean dynamicIsSet(InternalEObject owner, EStructuralFeature.Internal.DynamicValueHolder settings, int index)
     {
-      int featureID = owner.eClass().getEAllStructuralFeatures().indexOf(feature);
+      int featureID = owner.eClass().getFeatureID(feature);
       return owner.eContainer() != null && owner.eContainerFeatureID() == featureID;
     }
 
@@ -1720,8 +1730,8 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
       {
         notifications = owner.eBasicRemoveFromContainer(notifications);
       }
-      int featureID = owner.eClass().getEAllStructuralFeatures().indexOf(feature);
-      return owner.eBasicSetContainer((InternalEObject)otherEnd, featureID, notifications);
+      int featureID = owner.eClass().getFeatureID(feature);
+      return owner.eBasicSetContainer(otherEnd, featureID, notifications);
     }
 
     public NotificationChain dynamicInverseRemove
@@ -1731,7 +1741,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
       {
         notifications = owner.eBasicRemoveFromContainer(notifications);
       }
-      int featureID = owner.eClass().getEAllStructuralFeatures().indexOf(feature);
+      int featureID = owner.eClass().getFeatureID(feature);
       return owner.eBasicSetContainer(null, featureID, notifications);
     }
   }
@@ -2147,7 +2157,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
             notifications = 
               internalEObject.eInverseRemove
                 (owner,
-                 internalEObject.eClass().getEAllStructuralFeatures().indexOf(inverseFeature),
+                 internalEObject.eClass().getFeatureID(inverseFeature),
                  null,
                  notifications);
           }
@@ -2157,7 +2167,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
             notifications =
               internalEObject.eInverseAdd
                 (owner,
-                 internalEObject.eClass().getEAllStructuralFeatures().indexOf(inverseFeature),
+                 internalEObject.eClass().getFeatureID(inverseFeature),
                  null,
                  notifications);
           }
@@ -2226,7 +2236,6 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
     public void dynamicUnset(InternalEObject owner, EStructuralFeature.Internal.DynamicValueHolder settings, int index)
     {
       Object oldValue = settings.dynamicGet(index);
-      boolean oldIsSet = oldValue != null;
       if (isUnsettable() && oldValue == NIL)
       {
         oldValue = null;
@@ -2241,7 +2250,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
           notifications = 
             internalEObject.eInverseRemove
               (owner,
-               internalEObject.eClass().getEAllStructuralFeatures().indexOf(inverseFeature),
+               internalEObject.eClass().getFeatureID(inverseFeature),
                null,
                notifications);
         }
@@ -2321,7 +2330,7 @@ public abstract class EStructuralFeatureImpl extends ETypedElementImpl implement
           notifications = 
             internalEObject.eInverseRemove
               (owner,
-               internalEObject.eClass().getEAllStructuralFeatures().indexOf(inverseFeature),
+               internalEObject.eClass().getFeatureID(inverseFeature),
                null,
                notifications);
         }
