@@ -3,16 +3,16 @@
  *
  * Copyright (c) 2002-2004 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors:
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: EcoreEditor.java,v 1.12 2004/07/16 16:25:59 marcelop Exp $
+ * $Id: EcoreEditor.java,v 1.19 2005/06/10 21:02:54 emerks Exp $
  */
 package org.eclipse.emf.ecore.presentation;
 
@@ -22,6 +22,7 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.EventObject;
+import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 
@@ -36,7 +37,7 @@ import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.Platform;
+import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.jface.action.IMenuListener;
 import org.eclipse.jface.action.IMenuManager;
 import org.eclipse.jface.action.IStatusLineManager;
@@ -45,7 +46,7 @@ import org.eclipse.jface.action.MenuManager;
 import org.eclipse.jface.action.Separator;
 import org.eclipse.jface.dialogs.MessageDialog;
 import org.eclipse.jface.dialogs.ProgressMonitorDialog;
-import org.eclipse.jface.viewers.ColumnWeightData;
+//import org.eclipse.jface.viewers.ColumnWeightData;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.ISelectionChangedListener;
 import org.eclipse.jface.viewers.ISelectionProvider;
@@ -54,25 +55,27 @@ import org.eclipse.jface.viewers.ListViewer;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.jface.viewers.StructuredSelection;
 import org.eclipse.jface.viewers.StructuredViewer;
-import org.eclipse.jface.viewers.TableLayout;
-import org.eclipse.jface.viewers.TableTreeViewer;
+//import org.eclipse.jface.viewers.TableLayout;
 import org.eclipse.jface.viewers.TableViewer;
 import org.eclipse.jface.viewers.TreeViewer;
 import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.custom.TableTree;
 import org.eclipse.swt.custom.CTabFolder;
 import org.eclipse.swt.dnd.DND;
 import org.eclipse.swt.dnd.Transfer;
 import org.eclipse.swt.events.ControlAdapter;
 import org.eclipse.swt.events.ControlEvent;
 import org.eclipse.swt.graphics.Point;
+//import org.eclipse.swt.layout.FillLayout;
+
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Menu;
-import org.eclipse.swt.widgets.Table;
-import org.eclipse.swt.widgets.TableColumn;
+//import org.eclipse.swt.widgets.Table;
+//import org.eclipse.swt.widgets.TableColumn;
 import org.eclipse.swt.widgets.Tree;
+//import org.eclipse.swt.widgets.TreeColumn;
+
 import org.eclipse.ui.IActionBars;
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IEditorPart;
@@ -84,7 +87,6 @@ import org.eclipse.ui.PartInitException;
 import org.eclipse.ui.actions.WorkspaceModifyOperation;
 import org.eclipse.ui.dialogs.SaveAsDialog;
 import org.eclipse.ui.ide.IGotoMarker;
-
 import org.eclipse.ui.part.FileEditorInput;
 import org.eclipse.ui.part.MultiPageEditorPart;
 import org.eclipse.ui.views.contentoutline.ContentOutline;
@@ -94,6 +96,7 @@ import org.eclipse.ui.views.properties.IPropertySheetPage;
 import org.eclipse.ui.views.properties.PropertySheet;
 import org.eclipse.ui.views.properties.PropertySheetPage;
 
+import org.eclipse.emf.common.CommonPlugin;
 import org.eclipse.emf.common.command.BasicCommandStack;
 import org.eclipse.emf.common.command.Command;
 import org.eclipse.emf.common.command.CommandStack;
@@ -101,15 +104,14 @@ import org.eclipse.emf.common.command.CommandStackListener;
 import org.eclipse.emf.common.notify.AdapterFactory;
 import org.eclipse.emf.common.ui.ViewerPane;
 import org.eclipse.emf.common.ui.viewer.IViewerProvider;
-
 import org.eclipse.emf.common.util.URI;
+import org.eclipse.emf.ecore.EObject;
+import org.eclipse.emf.ecore.EValidator;
+import org.eclipse.emf.ecore.plugin.EcorePlugin;
 import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
-import java.util.HashMap;
-
-import org.eclipse.core.runtime.NullProgressMonitor;
-
 import org.eclipse.emf.ecore.resource.Resource;
 import org.eclipse.emf.ecore.resource.ResourceSet;
+import org.eclipse.emf.ecore.xmi.impl.GenericXMLResourceFactoryImpl;
 import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
 import org.eclipse.emf.edit.domain.EditingDomain;
 import org.eclipse.emf.edit.domain.IEditingDomainProvider;
@@ -125,10 +127,6 @@ import org.eclipse.emf.edit.ui.dnd.ViewerDragAdapter;
 import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
 import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
 
-import org.eclipse.emf.ecore.EObject;
-import org.eclipse.emf.ecore.EValidator;
-import org.osgi.framework.Bundle;
-
 
 /**
  * This is an example of a Ecore model editor.
@@ -146,16 +144,9 @@ public class EcoreEditor
     {
       try
       {
-        Bundle plugin = Platform.getBundle("org.eclipse.xsd.editor");
-        Class theClass = 
-          plugin.loadClass
-            ("org.eclipse.xsd.presentation.XSDEditor$GenericXMLResourceFactoryImpl");
-        Object genericXMLResourceFactory = theClass.newInstance();
-        editingDomain.getResourceSet().getResourceFactoryRegistry().getExtensionToFactoryMap().put("*", genericXMLResourceFactory);
+        editingDomain.getResourceSet().getResourceFactoryRegistry().getExtensionToFactoryMap().put("*", new GenericXMLResourceFactoryImpl());
 
-        Class theItemProviderClass =
-          plugin.loadClass
-            ("org.eclipse.xsd.provider.XSDItemProviderAdapterFactory");
+        Class theItemProviderClass = CommonPlugin.loadClass("org.eclipse.xsd.edit", "org.eclipse.xsd.provider.XSDItemProviderAdapterFactory");
         AdapterFactory xsdItemProviderAdapterFactory = (AdapterFactory)theItemProviderClass.newInstance();
         adapterFactory.insertAdapterFactory(xsdItemProviderAdapterFactory);
       }
@@ -164,6 +155,21 @@ public class EcoreEditor
         EcoreEditorPlugin.INSTANCE.log(exception);
       }
     }
+    public void createModel()
+    {
+      super.createModel();
+      
+      // Load the schema and packages that were used to load the instance into this resource set.
+      //
+      ResourceSet resourceSet = editingDomain.getResourceSet();
+      Resource resource = (Resource)resourceSet.getResources().get(0);
+      EObject rootObject = (EObject)resource.getContents().get(0);
+      Resource metaDataResource =  rootObject.eClass().eResource();
+      if (metaDataResource != null && metaDataResource.getResourceSet() != null)
+      {
+        resourceSet.getResources().addAll(metaDataResource.getResourceSet().getResources());
+      }
+    }
   }
 
   /**
@@ -258,13 +264,12 @@ public class EcoreEditor
   protected TableViewer tableViewer;
 
   /**
-   * This shows how a table view works.
-   * A table can be used as a list with icons.
+   * This shows how a tree view with columns works.
    * <!-- begin-user-doc -->
    * <!-- end-user-doc -->
    * @generated
    */
-  protected TableTreeViewer tableTreeViewer;
+  protected TreeViewer treeViewerWithColumns;
 
   /**
    * This keeps track of the active viewer pane, in the book.
@@ -382,7 +387,6 @@ public class EcoreEditor
     {
       public void resourceChanged(IResourceChangeEvent event)
       {
-        ResourceSet resourceSet = editingDomain.getResourceSet();
         // Only listening to these.
         // if (event.getType() == IResourceDelta.POST_CHANGE)
         {
@@ -434,21 +438,27 @@ public class EcoreEditor
             ResourceDeltaVisitor visitor = new ResourceDeltaVisitor();
             delta.accept(visitor);
 
-            removedResources.addAll(visitor.getRemovedResources());
-            if (!visitor.getRemovedResources().isEmpty() && !isDirty())
+            if (!visitor.getRemovedResources().isEmpty())
             {
-              getSite().getShell().getDisplay().asyncExec
-                (new Runnable()
-                 {
-                  public void run()
-                  {
-                    getSite().getPage().closeEditor(EcoreEditor.this, false);
-                    EcoreEditor.this.dispose();
-                  }
-                 });
+              removedResources.addAll(visitor.getRemovedResources());
+              if (!isDirty())
+              {
+                getSite().getShell().getDisplay().asyncExec
+                  (new Runnable()
+                   {
+                     public void run()
+                     {
+                       getSite().getPage().closeEditor(EcoreEditor.this, false);
+                       EcoreEditor.this.dispose();
+                     }
+                   });
+              }
             }
 
-            changedResources.addAll(visitor.getChangedResources());
+            if (!visitor.getChangedResources().isEmpty())
+            {
+              changedResources.addAll(visitor.getChangedResources());
+            }
           }
           catch (CoreException exception)
           {
@@ -802,21 +812,21 @@ public class EcoreEditor
    * This is the method called to load a resource into the editing domain's resource set based on the editor's input.
    * <!-- begin-user-doc -->
    * <!-- end-user-doc -->
-   * @generated
+   * @generated NOT
    */
   public void createModel()
   {
     // I assume that the input is a file object.
     //
     IFileEditorInput modelFile = (IFileEditorInput)getEditorInput();
+    
+    editingDomain.getResourceSet().getURIConverter().getURIMap().putAll(EcorePlugin.computePlatformURIMap());
 
     try
     {
       // Load the resource through the editing domain.
       //
-      Resource resource =
-        editingDomain.loadResource
-          (URI.createPlatformResourceURI(modelFile.getFile().getFullPath().toString()).toString());
+      editingDomain.loadResource(URI.createPlatformResourceURI(modelFile.getFile().getFullPath().toString()).toString());
     }
     catch (Exception exception)
     {
@@ -868,16 +878,6 @@ public class EcoreEditor
       setPageText(pageIndex, getString("_UI_SelectionPage_label"));
     }
     
-    ColumnWeightData junk1 = null;
-    TableLayout junk2 = null;
-    TableTree junk3 = null;
-    Table junk4 = null;
-    TableColumn junk5 = null;
-    if (junk1 == null && junk2 == null && junk3 == null && junk4 == null && junk5 == null)
-    {
-      // The compiler is happy that we used all the imports and the junky variables too.
-    }
-
     setActivePage(0);
 
     getContainer().addControlListener
