@@ -3,16 +3,16 @@
  *
  * Copyright (c) 2003-2004 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors:
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: BasicExtendedMetaData.java,v 1.11 2004/08/20 23:49:41 marcelop Exp $
+ * $Id: BasicExtendedMetaData.java,v 1.20 2005/06/08 06:20:10 nickb Exp $
  */
 package org.eclipse.emf.ecore.util;
 
@@ -20,8 +20,8 @@ package org.eclipse.emf.ecore.util;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.Hashtable;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.StringTokenizer;
@@ -51,6 +51,8 @@ public class BasicExtendedMetaData implements ExtendedMetaData
   protected String annotationURI;
   protected EPackage.Registry registry;
   protected EPackage.Registry demandRegistry;
+  protected Map extendedMetaDataHolderCache;
+  protected Map annotationMap;
 
   public BasicExtendedMetaData()
   {
@@ -64,29 +66,54 @@ public class BasicExtendedMetaData implements ExtendedMetaData
 
   public BasicExtendedMetaData(String annotationURI, EPackage.Registry registry)
   {
-    this.annotationURI = annotationURI;
-    this.registry = registry;
-    this.demandRegistry = new org.eclipse.emf.ecore.impl.EPackageRegistryImpl();
-    // init();
+    this(annotationURI, registry, null);
   }
 
+  public BasicExtendedMetaData(String annotationURI, EPackage.Registry registry, Map annotationMap)
+  {
+    this.annotationURI = annotationURI.intern();
+    this.registry = registry;
+    this.demandRegistry = new org.eclipse.emf.ecore.impl.EPackageRegistryImpl();
+    this.annotationMap = annotationMap;
+    
+    if (annotationURI != ANNOTATION_URI)
+    {
+      extendedMetaDataHolderCache = new HashMap();
+    }
+  }
+  
   protected EAnnotation getAnnotation(EModelElement eModelElement, boolean demandCreate)
   {
-    EAnnotation result = eModelElement.getEAnnotation(annotationURI);
-    if (result == null && demandCreate)
+    if (annotationMap != null)
     {
-      result = EcoreFactory.eINSTANCE.createEAnnotation();
-      result.setSource(annotationURI);
-      eModelElement.getEAnnotations().add(result);
+      EAnnotation result = (EAnnotation)annotationMap.get(eModelElement);
+      if (result == null && demandCreate)
+      {
+        result = EcoreFactory.eINSTANCE.createEAnnotation();
+        result.setSource(annotationURI);
+        annotationMap.put(eModelElement, result);
+      }
+      return result;
     }
-    return result;
+    else
+    {
+      EAnnotation result = eModelElement.getEAnnotation(annotationURI);
+      if (result == null && demandCreate)
+      {
+        result = EcoreFactory.eINSTANCE.createEAnnotation();
+        result.setSource(annotationURI);
+        eModelElement.getEAnnotations().add(result);
+      }
+      return result;
+    } 
   }
 
   public EClassifier getType(EPackage ePackage, String name)
   {
-    for (Iterator i = ePackage.getEClassifiers().iterator(); i.hasNext(); )
+    List eClassifiers = ePackage.getEClassifiers();
+    for (int i = 0, size = eClassifiers.size(); i < size; ++i)
     {
-      EClassifier eClassifier = (EClassifier)i.next();
+      EClassifier eClassifier = (EClassifier)eClassifiers.get(i);
       if (name.equals(getName(eClassifier)))
       {
         return eClassifier;
@@ -127,9 +154,10 @@ public class BasicExtendedMetaData implements ExtendedMetaData
   {
     if (getContentKind(eClass) == MIXED_CONTENT)
     {
-      for (Iterator i = eClass.getEAllReferences().iterator(); i.hasNext(); )
+      List eAllReferences = eClass.getEAllReferences();
+      for (int i = 0, size = eAllReferences.size();  i < size; ++i)
       {
-        EReference eReference = (EReference)i.next();
+        EReference eReference = (EReference)eAllReferences.get(i);
         if ("xmlns:prefix".equals(getName(eReference)))
         {
           return eReference;
@@ -144,9 +172,10 @@ public class BasicExtendedMetaData implements ExtendedMetaData
   {
     if (getContentKind(eClass) == MIXED_CONTENT)
     {
-      for (Iterator i = eClass.getEAllReferences().iterator(); i.hasNext(); )
+      List eAllReferences = eClass.getEAllReferences();
+      for (int i = 0, size = eAllReferences.size();  i < size; ++i)
       {
-        EReference eReference = (EReference)i.next();
+        EReference eReference = (EReference)eAllReferences.get(i);
         if ("xsi:schemaLocation".equals(getName(eReference)))
         {
           return eReference;
@@ -280,6 +309,12 @@ public class BasicExtendedMetaData implements ExtendedMetaData
     getExtendedMetaData(eClassifier).setName(name);
   }
 
+  public boolean isAnonymous(EClassifier eClassifier)
+  {
+    String name =  getExtendedMetaData(eClassifier).getName();
+    return name.length() == 0 || name.indexOf("_._") != -1;
+  }
+
   public String getName(EStructuralFeature eStructuralFeature)
   {
     return getExtendedMetaData(eStructuralFeature).getName();
@@ -638,9 +673,9 @@ public class BasicExtendedMetaData implements ExtendedMetaData
       EAnnotation eAnnotation = getAnnotation(eDataType, true);
       String namespace = getNamespace(eDataType);
       StringBuffer result = new StringBuffer();
-      for (Iterator i = memberTypes.iterator(); i.hasNext(); )
+      for (int i = 0, size = memberTypes.size(); i < size; ++i)
       {
-        result.append(getQualifiedName(namespace, (EDataType)i.next()));
+        result.append(getQualifiedName(namespace, (EDataType)memberTypes.get(i)));
         result.append(' ');
       }
       eAnnotation.getDetails().put("memberTypes", result.substring(0, result.length() - 1));
@@ -648,15 +683,44 @@ public class BasicExtendedMetaData implements ExtendedMetaData
     getExtendedMetaData(eDataType).setMemberTypes(memberTypes);
   }
 
+  protected boolean isFeatureKindSpecific()
+  {
+    return true;
+  }
+
   public EStructuralFeature getLocalAttribute(EClass eClass, String namespace, String name)
   {
-    for (Iterator i = getAllAttributes(eClass).iterator(); i.hasNext(); )
+    if (isFeatureKindSpecific())
     {
-      EStructuralFeature eStructuralFeature = (EStructuralFeature)i.next();
-      if (name.equals(getName(eStructuralFeature)) &&
-            (namespace == null ? getNamespace(eStructuralFeature) == null : namespace.equals(getNamespace(eStructuralFeature))))
+      List allAttributes = getAllAttributes(eClass);
+      for (int i = 0, size = allAttributes.size(); i < size; ++i)
       {
-        return eStructuralFeature;
+        EStructuralFeature eStructuralFeature = (EStructuralFeature)allAttributes.get(i);
+        if (name.equals(getName(eStructuralFeature)) &&
+              (namespace == null ? getNamespace(eStructuralFeature) == null : namespace.equals(getNamespace(eStructuralFeature))))
+        {
+          return eStructuralFeature;
+        }
+      }
+    }
+    else
+    {
+      for (int i = 0, size = eClass.getFeatureCount(); i < size; ++i)
+      {
+        EStructuralFeature eStructuralFeature = eClass.getEStructuralFeature(i);
+        switch (getFeatureKind(eStructuralFeature))
+        {
+          case UNSPECIFIED_FEATURE:
+          case ATTRIBUTE_FEATURE:
+          {
+            if (name.equals(getName(eStructuralFeature)) &&
+                  (namespace == null ? getNamespace(eStructuralFeature) == null : namespace.equals(getNamespace(eStructuralFeature))))
+            {
+              return eStructuralFeature;
+            }
+            break;
+          }
+        }
       }
     }
 
@@ -679,13 +743,37 @@ public class BasicExtendedMetaData implements ExtendedMetaData
 
   protected EStructuralFeature getLocalElement(EClass eClass, String namespace, String name)
   {
-    for (Iterator i = getAllElements(eClass).iterator(); i.hasNext(); )
+    if (isFeatureKindSpecific())
     {
-      EStructuralFeature eStructuralFeature = (EStructuralFeature)i.next();
-      if (name.equals(getName(eStructuralFeature)) &&
-            (namespace == null ? getNamespace(eStructuralFeature) == null : namespace.equals(getNamespace(eStructuralFeature))))
+      List allElements = getAllElements(eClass);
+      for (int i = 0, size = allElements.size(); i < size; ++i)
       {
-        return eStructuralFeature;
+        EStructuralFeature eStructuralFeature = (EStructuralFeature)allElements.get(i);
+        if (name.equals(getName(eStructuralFeature)) &&
+              (namespace == null ? getNamespace(eStructuralFeature) == null : namespace.equals(getNamespace(eStructuralFeature))))
+        {
+          return eStructuralFeature;
+        }
+      }
+    }
+    else
+    {
+      for (int i = 0, size = eClass.getFeatureCount(); i < size; ++i)
+      {
+        EStructuralFeature eStructuralFeature = eClass.getEStructuralFeature(i);
+        switch (getFeatureKind(eStructuralFeature))
+        {
+          case UNSPECIFIED_FEATURE:
+          case ELEMENT_FEATURE:
+          {
+            if (name.equals(getName(eStructuralFeature)) &&
+                  (namespace == null ? getNamespace(eStructuralFeature) == null : namespace.equals(getNamespace(eStructuralFeature))))
+            {
+              return eStructuralFeature;
+            }
+            break;
+          }
+        }
       }
     }
 
@@ -705,27 +793,161 @@ public class BasicExtendedMetaData implements ExtendedMetaData
     }
     return result;
   }
-
+  
   public List /*EStructuralFeature*/ getAllAttributes(EClass eClass)
   {
-    List result = new ArrayList();
-    for (Iterator i = eClass.getESuperTypes().iterator(); i.hasNext(); )
+    List superTypes = eClass.getESuperTypes();
+    List result = null;
+    boolean changeable = false;
+    for (int i = 0, size = superTypes.size(); i < size; ++i) 
     {
-      EClass eSuperType = (EClass)i.next();
-      result.addAll(getAllAttributes(eSuperType));
+      EClass eSuperType = (EClass)superTypes.get(i);
+      List allAttributes =  getAllAttributes(eSuperType);
+      if (!allAttributes.isEmpty())
+      {
+        if (result == null)
+        {
+          result = allAttributes;
+        }
+        else
+        {
+          if (!changeable)
+          {
+            changeable = true;
+            result = new UniqueEList(result);
+          }
+          result.addAll(allAttributes);
+        }
+      }
     }
+    List attributes = getAttributes(eClass);
+    if (!attributes.isEmpty())
+    {
+      if (result == null)
+      {
+        return attributes;
+      }
+      else
+      {
+        if (!changeable)
+        {
+          result = new UniqueEList(result);
+        }
+        result.addAll(attributes);
+        return result;
+      }
+    }
+    else
+    {
+      return result == null ? Collections.EMPTY_LIST : result;
+    }
+  }
+  
+  public List /*EStructuralFeature*/ getAllElements(EClass eClass)
+  {
+    List superTypes = eClass.getESuperTypes();
+    List result = null;
+    boolean changeable = false;
+    for (int i = 0, size = superTypes.size(); i < size; ++i) 
+    {
+      EClass eSuperType = (EClass)superTypes.get(i);
+      List allElements =  getAllElements(eSuperType);
+      if (!allElements.isEmpty())
+      {
+        if (result == null)
+        {
+          result = allElements;
+        }
+        else
+        {
+          if (!changeable)
+          {
+            changeable = true;
+            result = new UniqueEList(result);
+          }
+          result.addAll(allElements);
+        }
+      }
+    }
+    List elements = getElements(eClass);
+    if (!elements.isEmpty())
+    {
+      if (result == null)
+      {
+        return elements;
+      }
+      else
+      {
+        if (!changeable)
+        {
+          result = new UniqueEList(result);
+        }
+        result.addAll(elements);
+        return result;
+      }
+    }
+    else
+    {
+      return result == null ? Collections.EMPTY_LIST : result;
+    }
+  }
 
-    result.addAll(getAttributes(eClass));
-    return result;
+  public List /*EStructuralFeature*/ getAttributes(EClass eClass)
+  {
+    List eStructuralFeatures = eClass.getEStructuralFeatures();
+    List result = null;
+    for (int i = 0, size = eStructuralFeatures.size(); i < size; ++i)
+    {
+      EStructuralFeature eStructuralFeature = (EStructuralFeature)eStructuralFeatures.get(i);
+      switch (getFeatureKind(eStructuralFeature))
+      {
+        case ATTRIBUTE_FEATURE:
+        case ATTRIBUTE_WILDCARD_FEATURE:
+        {
+          if (result == null)
+          {
+            result = new ArrayList();
+          }
+          result.add(eStructuralFeature);
+        }
+      }
+    }
+    return result == null ? Collections.EMPTY_LIST : result;
+  }
+
+  public List /*EStructuralFeature*/ getElements(EClass eClass)
+  {
+    List eStructuralFeatures = eClass.getEStructuralFeatures();
+    List result = null;
+    for (int i = 0, size = eStructuralFeatures.size(); i < size; ++i)
+    {
+      EStructuralFeature eStructuralFeature = (EStructuralFeature)eStructuralFeatures.get(i);
+      switch (getFeatureKind(eStructuralFeature))
+      {
+        case ELEMENT_FEATURE:
+        case ELEMENT_WILDCARD_FEATURE:
+        case GROUP_FEATURE:
+        {
+          if (result == null)
+          {
+            result = new ArrayList();
+          }
+          result.add(eStructuralFeature);
+          break;
+        }
+      }
+    }
+    
+   return result == null ? Collections.EMPTY_LIST : result;
   }
 
   public EStructuralFeature getSimpleFeature(EClass eClass)
   {
     if (getContentKind(eClass) == SIMPLE_CONTENT)
     {
-      for (Iterator i = eClass.getEAllStructuralFeatures().iterator(); i.hasNext(); )
+      for (int i = 0, size = eClass.getFeatureCount(); i < size; ++i)
       {
-        EStructuralFeature eStructuralFeature = (EStructuralFeature)i.next();
+        EStructuralFeature eStructuralFeature = eClass.getEStructuralFeature(i);
         if (getFeatureKind(eStructuralFeature) == ExtendedMetaData.SIMPLE_FEATURE)
         {
           return eStructuralFeature;
@@ -743,9 +965,10 @@ public class BasicExtendedMetaData implements ExtendedMetaData
       case MIXED_CONTENT:
       case SIMPLE_CONTENT:
       {
-        for (Iterator i = eClass.getEAllAttributes().iterator(); i.hasNext(); )
+        List eAllAttributes = eClass.getEAllAttributes();
+        for (int i = 0, size = eAllAttributes.size(); i < size; ++i)
         {
-          EAttribute eAttribute = (EAttribute)i.next();
+          EAttribute eAttribute = (EAttribute)eAllAttributes.get(i);
           if (getFeatureKind(eAttribute) == ExtendedMetaData.ELEMENT_WILDCARD_FEATURE)
           {
             return eAttribute;
@@ -758,63 +981,6 @@ public class BasicExtendedMetaData implements ExtendedMetaData
     return null;
   }
 
-  public List /*EStructuralFeature*/ getAllElements(EClass eClass)
-  {
-    // REVISIT: this may affect performance. 
-    // In general, there is no multiple inheritance. 
-    // The multiple inheritance case handled here 
-    // is for org.eclipse.emf.ecore.sdo.EDataObjectSimpleAnyType
-    List result = new UniqueEList();
-    for (Iterator i = eClass.getESuperTypes().iterator(); i.hasNext(); )
-    {
-      EClass eSuperType = (EClass)i.next();
-      result.addAll(getAllElements(eSuperType));
-    }
-
-    result.addAll(getElements(eClass));
-    return result;
-  }
-
-  public List /*EStructuralFeature*/ getAttributes(EClass eClass)
-  {
-    List result = new ArrayList();
-    for (Iterator i = eClass.getEStructuralFeatures().iterator(); i.hasNext(); )
-    {
-      EStructuralFeature eStructuralFeature = (EStructuralFeature)i.next();
-      switch (getFeatureKind(eStructuralFeature))
-      {
-        case ATTRIBUTE_FEATURE:
-        case ATTRIBUTE_WILDCARD_FEATURE:
-        {
-          result.add(eStructuralFeature);
-        }
-      }
-    }
-
-    return result;
-  }
-
-  public List /*EStructuralFeature*/ getElements(EClass eClass)
-  {
-    List result = new ArrayList();
-    for (Iterator i = eClass.getEStructuralFeatures().iterator(); i.hasNext(); )
-    {
-      EStructuralFeature eStructuralFeature = (EStructuralFeature)i.next();
-      switch (getFeatureKind(eStructuralFeature))
-      {
-        case ELEMENT_FEATURE:
-        case ELEMENT_WILDCARD_FEATURE:
-        case GROUP_FEATURE:
-        {
-          result.add(eStructuralFeature);
-          break;
-        }
-      }
-    }
-
-    return result;
-  }
-
   public List /*String*/ getWildcards(EStructuralFeature eStructuralFeature)
   {
     return getExtendedMetaData(eStructuralFeature).getWildcards();
@@ -871,10 +1037,24 @@ public class BasicExtendedMetaData implements ExtendedMetaData
     {
       String namespace = getNamespace(eStructuralFeature.getEContainingClass().getEPackage());
       EAnnotation eAnnotation = getAnnotation(eStructuralFeature, true);
+      eAnnotation.getDetails().put("wildcards", getEncodedWildcards(namespace, wildcards));
+      eAnnotation.getDetails().put("name","");
+    }
+    getExtendedMetaData(eStructuralFeature).setWildcards(wildcards);
+  }
+  
+  public static String getEncodedWildcards(String namespace, List wildcards)
+  {
+    if (wildcards.isEmpty())
+    {
+      return ""; 
+    }
+    else
+    {
       StringBuffer value = new StringBuffer();
-      for (Iterator i = wildcards.iterator(); i.hasNext(); )
+      for (int i = 0, size = wildcards.size(); i < size; )
       {
-        String wildcard = (String)i.next();
+        String wildcard = (String)wildcards.get(i);
         if (wildcard == null)
         {
           if (namespace == null)
@@ -907,15 +1087,14 @@ public class BasicExtendedMetaData implements ExtendedMetaData
         {
           value.append(wildcard);
         }
-        if (i.hasNext())
+
+        if (++i < size)
         {
           value.append(' ');
         }
       }
-      eAnnotation.getDetails().put("wildcards", value.toString());
-      eAnnotation.getDetails().put("name","");
+      return value.toString();
     }
-    getExtendedMetaData(eStructuralFeature).setWildcards(wildcards);
   }
 
   public int getProcessingKind(EStructuralFeature eStructuralFeature)
@@ -1076,8 +1255,7 @@ public class BasicExtendedMetaData implements ExtendedMetaData
 
   public EStructuralFeature getAffiliation(EClass eClass, EStructuralFeature eStructuralFeature)
   {
-    List eAllStructuralFeatures = eClass.getEAllStructuralFeatures();
-    if (eAllStructuralFeatures.contains(eStructuralFeature))
+    if (eClass.getFeatureID(eStructuralFeature) >= 0) 
     {
       return eStructuralFeature;
     }
@@ -1094,9 +1272,10 @@ public class BasicExtendedMetaData implements ExtendedMetaData
           return result;
         }
 
-        for (Iterator i = getAllAttributes(eClass).iterator(); i.hasNext(); )
+        List allAttributes = getAllAttributes(eClass);
+        for (int i = 0, size = allAttributes.size(); i < size; ++i)
         {
-          result = (EStructuralFeature)i.next();
+          result = (EStructuralFeature)allAttributes.get(i);
           if (matches(getWildcards(result), namespace))
           {
             return result;
@@ -1125,9 +1304,10 @@ public class BasicExtendedMetaData implements ExtendedMetaData
         }
         else
         {
-          for (Iterator i = getAllElements(eClass).iterator(); i.hasNext(); )
+          List allElements = getAllElements(eClass);
+          for (int i = 0, size = allElements.size(); i < size; ++i)
           {
-            EStructuralFeature result = (EStructuralFeature)i.next();
+            EStructuralFeature result = (EStructuralFeature)allElements.get(i);
             if (matches(getWildcards(result), namespace))
             {
               return result;
@@ -1145,9 +1325,10 @@ public class BasicExtendedMetaData implements ExtendedMetaData
 
   public EStructuralFeature getAttributeWildcardAffiliation(EClass eClass, String namespace, String name)
   {
-    for (Iterator i = getAllAttributes(eClass).iterator(); i.hasNext(); )
+    List allAttributes = getAllAttributes(eClass);
+    for (int i = 0, size = allAttributes.size(); i < size; ++i)
     {
-      EStructuralFeature result = (EStructuralFeature)i.next();
+      EStructuralFeature result = (EStructuralFeature)allAttributes.get(i);
       if (matches(getWildcards(result), namespace))
       {
         return result;
@@ -1159,9 +1340,10 @@ public class BasicExtendedMetaData implements ExtendedMetaData
 
   public EStructuralFeature getElementWildcardAffiliation(EClass eClass, String namespace, String name)
   {
-    for (Iterator i = getAllElements(eClass).iterator(); i.hasNext(); )
+    List allElements = getAllElements(eClass);
+    for (int i = 0, size = allElements.size(); i < size; ++i)
     {
-      EStructuralFeature result = (EStructuralFeature)i.next();
+      EStructuralFeature result = (EStructuralFeature)allElements.get(i);
       if (matches(getWildcards(result), namespace))
       {
         return result;
@@ -1175,9 +1357,9 @@ public class BasicExtendedMetaData implements ExtendedMetaData
   {
     if (!wildcards.isEmpty())
     {
-      for (Iterator j = wildcards.iterator(); j.hasNext(); )
+      for (int i = 0, size = wildcards.size(); i < size; ++i)
       {
-        String wildcard = (String)j.next();
+        String wildcard = (String)wildcards.get(i);
         if (matches(wildcard, namespace))
         {
           return true;
@@ -1280,9 +1462,9 @@ public class BasicExtendedMetaData implements ExtendedMetaData
     {
       EAnnotation eAnnotation = getAnnotation(eDataType, true);
       StringBuffer result = new StringBuffer();
-      for (Iterator i = literals.iterator(); i.hasNext(); )
+      for (int i = 0, size = literals.size(); i < size; ++i)
       {
-        result.append(replace(replace(((String)i.next()), "%","%25"), " ", "%20"));
+        result.append(replace(replace(((String)literals.get(i)), "%","%25"), " ", "%20"));
         result.append(' ');
       }
       eAnnotation.getDetails().put("enumeration", result.substring(0, result.length() - 1));
@@ -1329,9 +1511,9 @@ public class BasicExtendedMetaData implements ExtendedMetaData
     {
       EAnnotation eAnnotation = getAnnotation(eDataType, true);
       StringBuffer result = new StringBuffer();
-      for (Iterator i = pattern.iterator(); i.hasNext(); )
+      for (int i = 0, size = pattern.size(); i < size; ++i)
       {
-        result.append(replace(replace(((String)i.next()), "%","%25"), " ", "%20"));
+        result.append(replace(replace(((String)pattern.get(i)), "%","%25"), " ", "%20"));
         result.append(' ');
       }
       eAnnotation.getDetails().put("pattern", result.substring(0, result.length() - 1));
@@ -1662,7 +1844,12 @@ public class BasicExtendedMetaData implements ExtendedMetaData
       setQualified(ePackage, namespace != null);
       if (namespace != null)
       {
-        ePackage.setNsPrefix("_");
+        ePackage.setNsPrefix
+          (namespace.equals(ExtendedMetaData.XMLNS_URI) ? 
+             namespace.equals(ExtendedMetaData.XML_URI) ?
+               "xml" : 
+               "xmlns" : 
+             "_");
       }
       demandRegistry.put(namespace, ePackage);
 
@@ -1815,13 +2002,25 @@ public class BasicExtendedMetaData implements ExtendedMetaData
 
   protected EPackageExtendedMetaData getExtendedMetaData(EPackage ePackage)
   {
-    EPackageExtendedMetaData.Holder holder = (EPackageExtendedMetaData.Holder)ePackage;
-    EPackageExtendedMetaData result = holder.getExtendedMetaData();
-    if (result == null)
+    if (extendedMetaDataHolderCache != null)
     {
-      holder.setExtendedMetaData(result = createEPackageExtendedMetaData(ePackage));
+      EPackageExtendedMetaData result = (EPackageExtendedMetaData)extendedMetaDataHolderCache.get(ePackage);
+      if (result == null)
+      {
+        extendedMetaDataHolderCache.put(ePackage, result = createEPackageExtendedMetaData(ePackage));
+      }
+      return result;
     }
-    return result;
+    else
+    {
+      EPackageExtendedMetaData.Holder holder = (EPackageExtendedMetaData.Holder)ePackage;
+      EPackageExtendedMetaData result = holder.getExtendedMetaData();
+      if (result == null)
+      {
+        holder.setExtendedMetaData(result = createEPackageExtendedMetaData(ePackage));
+      }
+      return result;
+    } 
   }
 
   protected EPackageExtendedMetaData createEPackageExtendedMetaData(EPackage ePackage)
@@ -2382,13 +2581,25 @@ public class BasicExtendedMetaData implements ExtendedMetaData
 
   protected EClassifierExtendedMetaData getExtendedMetaData(EClassifier eClassifier)
   {
-    EClassifierExtendedMetaData.Holder holder = (EClassifierExtendedMetaData.Holder)eClassifier;
-    EClassifierExtendedMetaData result = holder.getExtendedMetaData();
-    if (result == null)
+    if (extendedMetaDataHolderCache != null)
     {
-      holder.setExtendedMetaData(result = createEClassifierExtendedMetaData(eClassifier));
+      EClassifierExtendedMetaData result = (EClassifierExtendedMetaData)extendedMetaDataHolderCache.get(eClassifier);
+      if (result == null)
+      {
+        extendedMetaDataHolderCache.put(eClassifier, result = createEClassifierExtendedMetaData(eClassifier));
+      }
+      return result;
     }
-    return result;
+    else
+    {
+      EClassifierExtendedMetaData.Holder holder = (EClassifierExtendedMetaData.Holder)eClassifier;
+      EClassifierExtendedMetaData result = holder.getExtendedMetaData();
+      if (result == null)
+      {
+        holder.setExtendedMetaData(result = createEClassifierExtendedMetaData(eClassifier));
+      }
+      return result;
+    }  
   }
 
   protected EClassifierExtendedMetaData createEClassifierExtendedMetaData(EClassifier eClassifier)
@@ -2562,13 +2773,25 @@ public class BasicExtendedMetaData implements ExtendedMetaData
 
   protected EStructuralFeatureExtendedMetaData getExtendedMetaData(EStructuralFeature eStructuralFeature)
   {
-    EStructuralFeatureExtendedMetaData.Holder holder = (EStructuralFeatureExtendedMetaData.Holder)eStructuralFeature;
-    EStructuralFeatureExtendedMetaData result = holder.getExtendedMetaData();
-    if (result == null)
+    if (extendedMetaDataHolderCache != null)
     {
-      holder.setExtendedMetaData(result = createEStructuralFeatureExtendedMetaData(eStructuralFeature));
+      EStructuralFeatureExtendedMetaData result = (EStructuralFeatureExtendedMetaData)extendedMetaDataHolderCache.get(eStructuralFeature);
+      if (result == null)
+      {
+        extendedMetaDataHolderCache.put(eStructuralFeature, result = createEStructuralFeatureExtendedMetaData(eStructuralFeature));
+      }
+      return result;
+    }
+    else
+    {
+      EStructuralFeatureExtendedMetaData.Holder holder = (EStructuralFeatureExtendedMetaData.Holder)eStructuralFeature;
+      EStructuralFeatureExtendedMetaData result = holder.getExtendedMetaData();
+      if (result == null)
+      {
+        holder.setExtendedMetaData(result = createEStructuralFeatureExtendedMetaData(eStructuralFeature));
+      }
+      return result;
     }
-    return result;
   }
 
   protected EStructuralFeatureExtendedMetaData createEStructuralFeatureExtendedMetaData(EStructuralFeature eStructuralFeature)
@@ -2578,39 +2801,41 @@ public class BasicExtendedMetaData implements ExtendedMetaData
 
   private static String replace(String in, String oldString, String newString)
   {
-    if((in == null) || (oldString == null))
+    if (in == null || oldString == null)
     {
       return in;
     }
       
     int oldStringLength = oldString.length();
-    if(oldStringLength == 0)
+    if (oldStringLength == 0)
     {
       return in;
     }
     
-    if(newString == null)
+    if (newString == null)
+    {
       newString = "";
+    }
     int newStringLength = newString.length();
       
-    int index = 0-newStringLength;
-    StringBuffer sbIn = new StringBuffer(in);
-    while((index=indexOf(sbIn, oldString, index+newStringLength)) >= 0)
+    int index = -newStringLength;
+    StringBuffer result = new StringBuffer(in);
+    while((index = indexOf(result, oldString, index + newStringLength)) >= 0)
     {
-      sbIn.replace(index, index+oldStringLength, newString);
+      result.replace(index, index + oldStringLength, newString);
     }
     
-    return sbIn.toString();
+    return result.toString();
   }
   
   private static int indexOf(StringBuffer in, String str, int fromIndex)
   {
-    if(in == null)
+    if (in == null)
     {
       return -1;
     }
     
-    if(str == null)
+    if (str == null)
     {
       str = "";
     }
@@ -2618,12 +2843,12 @@ public class BasicExtendedMetaData implements ExtendedMetaData
     int lengthIn = in.length();
     int lengthStr = str.length();
 
-    if(lengthIn < lengthStr)
+    if (lengthIn < lengthStr)
     {
       return -1;
     }
 
-    if(fromIndex > lengthIn)
+    if (fromIndex > lengthIn)
     {
       if (lengthIn == 0 && fromIndex == 0 && lengthStr == 0)
       {
@@ -2632,27 +2857,33 @@ public class BasicExtendedMetaData implements ExtendedMetaData
       return -1;
     }
     
-    if(fromIndex < 0)
+    if (fromIndex < 0)
+    {
       fromIndex = 0;
+    }
       
-    if(lengthStr == 0)
+    if (lengthStr == 0)
     {
       return fromIndex;
     }
       
     int strPos = 0;
-    for(int i=fromIndex; i<lengthIn; i++)
+    for (int i = fromIndex; i < lengthIn; i++)
     {
-      if(in.charAt(i) == str.charAt(strPos))
+      if (in.charAt(i) == str.charAt(strPos))
       {
         strPos++;
         if(strPos == lengthStr)
-          return i-lengthStr+1;
+        {
+          return i - lengthStr + 1;
+        }
       }
       else
+      {
         strPos = 0; 
+      }
     }
     
     return -1;
-  }  
+  }
 }
