@@ -1,18 +1,18 @@
 /**
  * <copyright>
  *
- * Copyright (c) 2004 IBM Corporation and others.
+ * Copyright (c) 2004-2005 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors: 
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: EObjectValidator.java,v 1.4 2004/06/19 21:36:23 emerks Exp $
+ * $Id: EObjectValidator.java,v 1.11 2005/06/08 06:20:10 nickb Exp $
  */
 package org.eclipse.emf.ecore.util;
 
@@ -70,6 +70,54 @@ public class EObjectValidator implements EValidator
   public static final int DATA_VALUE__TOTAL_DIGITS_IN_RANGE = 10;
   public static final int DATA_VALUE__FRACTION_DIGITS_IN_RANGE = 11;
 
+  /**
+   * @since 2.1.0
+   */
+  public static String getObjectLabel(EObject eObject, Map context)
+  {
+    if (context != null)
+    {
+      SubstitutionLabelProvider substitutionlabelProvider = (SubstitutionLabelProvider)context.get(SubstitutionLabelProvider.class);
+      if (substitutionlabelProvider != null)
+      {
+        return substitutionlabelProvider.getObjectLabel(eObject);
+      }
+    }
+    return EcoreUtil.getIdentification(eObject);
+  }
+
+  /**
+   * @since 2.1.0
+   */
+  public static String getFeatureLabel(EStructuralFeature eStructuralFeature, Map context)
+  {
+    if (context != null)
+    {
+      SubstitutionLabelProvider substitutionlabelProvider = (SubstitutionLabelProvider)context.get(SubstitutionLabelProvider.class);
+      if (substitutionlabelProvider != null)
+      {
+        return substitutionlabelProvider.getFeatureLabel(eStructuralFeature);
+      }
+    }
+    return eStructuralFeature.getName();
+  }
+
+  /**
+   * @since 2.1.0
+   */
+  public static String getValueLabel(EDataType eDataType, Object value, Map context)
+  {
+    if (context != null)
+    {
+      SubstitutionLabelProvider substitutionlabelProvider = (SubstitutionLabelProvider)context.get(SubstitutionLabelProvider.class);
+      if (substitutionlabelProvider != null)
+      {
+        return substitutionlabelProvider.getValueLabel(eDataType, value);
+      }
+    }
+    return EcoreUtil.convertToString(eDataType, value);
+  }
+
   public EObjectValidator()
   {
   }
@@ -81,39 +129,16 @@ public class EObjectValidator implements EValidator
 
   protected EValidator getRootEValidator(Map context)
   {
-    EValidator result = (EValidator)context.get(EValidator.class);
-    if (result == null)
+    if (context != null)
     {
-      result = Diagnostician.INSTANCE;
+      EValidator result = (EValidator)context.get(EValidator.class);
+      if (result != null)
+      {
+        return result;
+      }
     }
-   return result;
-  }
 
-  protected String getObjectLabel(EObject eObject, Map context)
-  {
-    SubstitutionLabelProvider substitutionlabelProvider = (SubstitutionLabelProvider)context.get(SubstitutionLabelProvider.class);
-    return 
-      substitutionlabelProvider == null ?
-        EcoreUtil.getIdentification(eObject) :
-        substitutionlabelProvider.getObjectLabel(eObject);
-  }
-
-  protected String getFeatureLabel(EStructuralFeature eStructuralFeature, Map context)
-  {
-    SubstitutionLabelProvider substitutionlabelProvider = (SubstitutionLabelProvider)context.get(SubstitutionLabelProvider.class);
-    return 
-      substitutionlabelProvider == null ?
-        eStructuralFeature.getName() :
-        substitutionlabelProvider.getFeatureLabel(eStructuralFeature);
-  }
-
-  protected String getValueLabel(EDataType eDataType, Object value, Map context)
-  {
-    SubstitutionLabelProvider substitutionlabelProvider = (SubstitutionLabelProvider)context.get(SubstitutionLabelProvider.class);
-    return 
-      substitutionlabelProvider == null ?
-        EcoreUtil.convertToString(eDataType, value) :
-        substitutionlabelProvider.getValueLabel(eDataType, value);
+    return Diagnostician.INSTANCE;
   }
 
   /**
@@ -129,11 +154,11 @@ public class EObjectValidator implements EValidator
 
   public boolean validate(EClass eClass, EObject eObject, DiagnosticChain diagnostics, Map context)
   {
-    if (eClass.eContainer() == getEPackage()) 
+    if (eClass.eContainer() == getEPackage())
     {
       return validate(eClass.getClassifierID(), eObject, diagnostics, context);
     }
-    else 
+    else
     {
       List eSuperTypes = eClass.getESuperTypes();
       return
@@ -148,7 +173,7 @@ public class EObjectValidator implements EValidator
     return classifierID != EcorePackage.EOBJECT || validate_EveryDefaultConstraint((EObject)object, diagnostics, context);
   }
 
-  public boolean validate_EveryDefaultConstraint(EObject object, DiagnosticChain theDiagnostics, Map context) 
+  public boolean validate_EveryDefaultConstraint(EObject object, DiagnosticChain theDiagnostics, Map context)
   {
     boolean result = validate_EveryMultiplicityConforms(object, theDiagnostics, context);
     if (result || theDiagnostics != null)
@@ -169,9 +194,10 @@ public class EObjectValidator implements EValidator
   public boolean validate_EveryMultiplicityConforms(EObject eObject, DiagnosticChain diagnostics, Map context)
   {
     boolean result = true;
-    for (Iterator i = eObject.eClass().getEAllStructuralFeatures().iterator(); i.hasNext(); )
+    EClass eClass = eObject.eClass();
+    for (int i = 0, size = eClass.getFeatureCount(); i < size; ++i)
     {
-      result &= validate_MultiplicityConforms(eObject, (EStructuralFeature)i.next(), diagnostics, context);
+      result &= validate_MultiplicityConforms(eObject, eClass.getEStructuralFeature(i), diagnostics, context);
       if (!result && diagnostics == null)
       {
         return false;
@@ -201,13 +227,13 @@ public class EObjectValidator implements EValidator
                  DIAGNOSTIC_SOURCE,
                  EOBJECT__EVERY_MULTIPCITY_CONFORMS,
                  EcorePlugin.INSTANCE.getString
-                   ("_UI_FeatureHasTooFewValues_diagnostic", 
-                    new Object [] 
-                      { 
-                        getFeatureLabel(eStructuralFeature, context), 
-                        getObjectLabel(eObject, context), 
-                        new Integer(size), 
-                        new Integer(lowerBound) 
+                   ("_UI_FeatureHasTooFewValues_diagnostic",
+                    new Object []
+                      {
+                        getFeatureLabel(eStructuralFeature, context),
+                        getObjectLabel(eObject, context),
+                        new Integer(size),
+                        new Integer(lowerBound)
                       }),
                  new Object [] { eObject, eStructuralFeature }));
           }
@@ -224,13 +250,13 @@ public class EObjectValidator implements EValidator
                  DIAGNOSTIC_SOURCE,
                  EOBJECT__EVERY_MULTIPCITY_CONFORMS,
                  EcorePlugin.INSTANCE.getString
-                   ("_UI_FeatureHasTooManyValues_diagnostic", 
-                    new Object [] 
-                      { 
-                        getFeatureLabel(eStructuralFeature, context), 
-                        getObjectLabel(eObject, context), 
-                        new Integer(size), 
-                        new Integer(upperBound) 
+                   ("_UI_FeatureHasTooManyValues_diagnostic",
+                    new Object []
+                      {
+                        getFeatureLabel(eStructuralFeature, context),
+                        getObjectLabel(eObject, context),
+                        new Integer(size),
+                        new Integer(upperBound)
                       }),
                  new Object [] { eObject, eStructuralFeature }));
           }
@@ -253,13 +279,13 @@ public class EObjectValidator implements EValidator
                    DIAGNOSTIC_SOURCE,
                    EOBJECT__EVERY_MULTIPCITY_CONFORMS,
                    EcorePlugin.INSTANCE.getString
-                     ("_UI_FeatureHasTooManyValues_diagnostic", 
-                      new Object [] 
-                        { 
-                          getFeatureLabel(eStructuralFeature, context), 
-                          getObjectLabel(eObject, context), 
-                          new Integer(size), 
-                          new Integer(upperBound) 
+                     ("_UI_FeatureHasTooManyValues_diagnostic",
+                      new Object []
+                        {
+                          getFeatureLabel(eStructuralFeature, context),
+                          getObjectLabel(eObject, context),
+                          new Integer(size),
+                          new Integer(upperBound)
                         }),
                    new Object [] { eObject, eStructuralFeature }));
             }
@@ -280,7 +306,7 @@ public class EObjectValidator implements EValidator
                DIAGNOSTIC_SOURCE,
                EOBJECT__EVERY_MULTIPCITY_CONFORMS,
                EcorePlugin.INSTANCE.getString
-                 ("_UI_RequiredFeatureMustBeSet_diagnostic", 
+                 ("_UI_RequiredFeatureMustBeSet_diagnostic",
                   new Object [] { getFeatureLabel(eStructuralFeature, context), getObjectLabel(eObject, context) }),
                new Object [] { eObject, eStructuralFeature }));
         }
@@ -307,12 +333,12 @@ public class EObjectValidator implements EValidator
                DIAGNOSTIC_SOURCE,
                EOBJECT__EVERY_PROXY_RESOLVES,
                EcorePlugin.INSTANCE.getString
-                 ("_UI_UnresolvedProxy_diagnostic", 
-                  new Object [] 
-                    { 
-                      getFeatureLabel(i.feature(), context), 
-                      getObjectLabel(eObject, context), 
-                      getObjectLabel(eCrossReferenceObject, context) 
+                 ("_UI_UnresolvedProxy_diagnostic",
+                  new Object []
+                    {
+                      getFeatureLabel(i.feature(), context),
+                      getObjectLabel(eObject, context),
+                      getObjectLabel(eCrossReferenceObject, context)
                     }),
                new Object [] { eObject, i.feature(), eCrossReferenceObject }));
         }
@@ -331,7 +357,7 @@ public class EObjectValidator implements EValidator
     for (EContentsEList.FeatureIterator i = (EContentsEList.FeatureIterator)eObject.eCrossReferences().iterator(); i.hasNext(); )
     {
       EObject eCrossReferenceObject = (EObject)i.next();
-      if (eCrossReferenceObject.eResource() == null && !eCrossReferenceObject.eIsProxy())
+      if (eCrossReferenceObject.eResource() == null && !eCrossReferenceObject.eIsProxy() && !i.feature().isTransient())
       {
         result = false;
         if (diagnostics != null)
@@ -342,12 +368,12 @@ public class EObjectValidator implements EValidator
                DIAGNOSTIC_SOURCE,
                EOBJECT__EVERY_REFERENCE_IS_CONTAINED,
                EcorePlugin.INSTANCE.getString
-                 ("_UI_DanglingReference_diagnostic", 
-                  new Object [] 
-                    { 
-                      getFeatureLabel(i.feature(), context), 
-                      getObjectLabel(eObject, context), 
-                      getObjectLabel(eCrossReferenceObject, context) 
+                 ("_UI_DanglingReference_diagnostic",
+                  new Object []
+                    {
+                      getFeatureLabel(i.feature(), context),
+                      getObjectLabel(eObject, context),
+                      getObjectLabel(eCrossReferenceObject, context)
                     }),
                new Object [] { eObject, i.feature(), eCrossReferenceObject }));
         }
@@ -453,7 +479,7 @@ public class EObjectValidator implements EValidator
   protected DiagnosticChain createBadDataValueDiagnostic
     (EObject eObject, EAttribute eAttribute, DiagnosticChain diagnostics, Map context)
   {
-    BasicDiagnostic diagnostic = 
+    BasicDiagnostic diagnostic =
       new BasicDiagnostic
         (Diagnostic.ERROR,
          DIAGNOSTIC_SOURCE,
@@ -500,18 +526,18 @@ public class EObjectValidator implements EValidator
 
   public class DynamicEDataTypeValidator
   {
-    protected List effectiveEnumeration; 
-    protected PatternMatcher [][] effectivePattern; 
-    protected int effectiveTotalDigits = -1; 
-    protected int effectiveFractionDigits = -1; 
-    protected int effectiveMinLength = -1; 
-    protected int effectiveMaxLength = -1; 
-    protected Object effectiveMin; 
+    protected List effectiveEnumeration;
+    protected PatternMatcher [][] effectivePattern;
+    protected int effectiveTotalDigits = -1;
+    protected int effectiveFractionDigits = -1;
+    protected int effectiveMinLength = -1;
+    protected int effectiveMaxLength = -1;
+    protected Object effectiveMin;
     protected boolean effectiveMinIsInclusive;
-    protected int effectiveTotalDigitsMin = -1; 
-    protected Object effectiveMax; 
+    protected int effectiveTotalDigitsMin = -1;
+    protected Object effectiveMax;
     protected boolean effectiveMaxIsInclusive;
-    protected int effectiveTotalDigitsMax = -1; 
+    protected int effectiveTotalDigitsMax = -1;
     protected EDataType itemType;
     protected List memberTypes;
 
@@ -653,7 +679,7 @@ public class EObjectValidator implements EValidator
         Object lowerBound = EcoreUtil.createFromString(eDataType, "-" + digits.toString());
 
         if (effectiveMin == null ||
-              (effectiveMinIsInclusive ? 
+              (effectiveMinIsInclusive ?
                  ((Comparable)effectiveMin).compareTo(lowerBound) <= 0:
                  ((Comparable)effectiveMin).compareTo(lowerBound) < 0))
         {
@@ -663,7 +689,7 @@ public class EObjectValidator implements EValidator
         }
 
         if (effectiveMax == null ||
-              (effectiveMaxIsInclusive ? 
+              (effectiveMaxIsInclusive ?
                  ((Comparable)effectiveMax).compareTo(upperBound) >= 0:
                  ((Comparable)effectiveMax).compareTo(upperBound) > 0))
         {
@@ -699,12 +725,12 @@ public class EObjectValidator implements EValidator
 
       if (effectiveMin != null)
       {
-        if (effectiveMinIsInclusive ? 
+        if (effectiveMinIsInclusive ?
               ((Comparable)effectiveMin).compareTo(value) > 0:
               ((Comparable)effectiveMin).compareTo(value) >= 0)
         {
-          if (diagnostics != null) 
-          { 
+          if (diagnostics != null)
+          {
             if (effectiveTotalDigitsMin != -1)
             {
               reportTotalDigitsViolation(eDataType, value, effectiveTotalDigitsMin, diagnostics, context);
@@ -720,11 +746,11 @@ public class EObjectValidator implements EValidator
 
       if (effectiveMax != null)
       {
-        if (effectiveMaxIsInclusive ? 
+        if (effectiveMaxIsInclusive ?
               ((Comparable)effectiveMax).compareTo(value) < 0:
               ((Comparable)effectiveMax).compareTo(value) <= 0)
         {
-          if (diagnostics != null) 
+          if (diagnostics != null)
           {
             if (effectiveTotalDigitsMax != -1)
             {
@@ -762,7 +788,7 @@ public class EObjectValidator implements EValidator
              value instanceof Object[] ?
                ((Object[])value).length :
                ((Collection)value).size();
-        if (length < effectiveMaxLength)
+        if (length > effectiveMaxLength)
         {
           if (diagnostics != null) reportMaxLengthViolation(eDataType, value, length, effectiveMaxLength, diagnostics, context);
           result = false;
@@ -839,13 +865,13 @@ public class EObjectValidator implements EValidator
       }
     }
 
-    if (eDataType.eContainer() == getEPackage()) 
+    if (eDataType.eContainer() == getEPackage())
     {
       return validate(eDataType.getClassifierID(), value, diagnostics, context);
     }
-    else 
+    else
     {
-      return 
+      return
         new DynamicEDataTypeValidator(eDataType)
         {
         }.validate(eDataType, value, diagnostics, context);
@@ -971,13 +997,13 @@ public class EObjectValidator implements EValidator
     Iterator i = values.iterator();
     if (i.hasNext())
     {
-      valueLiterals = 
+      valueLiterals =
         EcorePlugin.INSTANCE.getString("_UI_ListHead_composition", new Object [] { getValueLabel(eDataType, i.next(), context) });
       while (i.hasNext())
       {
-        valueLiterals = 
+        valueLiterals =
           EcorePlugin.INSTANCE.getString
-            ("_UI_ListTail_composition", 
+            ("_UI_ListTail_composition",
              new Object [] { valueLiterals, getValueLabel(eDataType, i.next(), context) });
       }
     }
