@@ -3,16 +3,16 @@
  *
  * Copyright (c) 2002-2004 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors: 
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: DragAndDropCommand.java,v 1.4 2004/07/29 13:33:00 marcelop Exp $
+ * $Id: DragAndDropCommand.java,v 1.6 2005/06/08 06:17:05 nickb Exp $
  */
 package org.eclipse.emf.edit.command;
 
@@ -496,7 +496,14 @@ public class DragAndDropCommand extends AbstractCommand implements DragAndDropFe
         Object object = objects.next();
         if (collection.contains(object))
         {
-          (j <= index ? before : after).add(object);
+          if (j < index)
+          {
+            before.add(object);
+          }
+          else if (j > index)
+          {
+            after.add(object);
+          }
         }
       }
 
@@ -512,7 +519,7 @@ public class DragAndDropCommand extends AbstractCommand implements DragAndDropFe
         compoundCommand.append(MoveCommand.create(domain, parent, null, object, index));
       }
 
-      dropCommand = compoundCommand;
+      dropCommand = compoundCommand.getCommandList().size() == 0 ? (Command)IdentityCommand.INSTANCE : compoundCommand;
     }
     else if (isCrossDomain())
     {
