@@ -3,16 +3,16 @@
  *
  * Copyright (c) 2002-2004 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors:
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: EcoreXMLSchemaBuilder.java,v 1.2 2004/03/21 16:22:54 emerks Exp $
+ * $Id: EcoreXMLSchemaBuilder.java,v 1.7 2005/06/08 06:23:01 nickb Exp $
  */
 package org.eclipse.xsd.ecore;
 
@@ -45,6 +45,7 @@ import org.eclipse.emf.ecore.util.EcoreUtil;
 import org.eclipse.xsd.XSDAnnotation;
 import org.eclipse.xsd.XSDAttributeDeclaration;
 import org.eclipse.xsd.XSDAttributeUse;
+import org.eclipse.xsd.XSDAttributeUseCategory;
 import org.eclipse.xsd.XSDComplexTypeDefinition;
 import org.eclipse.xsd.XSDCompositor;
 import org.eclipse.xsd.XSDConcreteComponent;
@@ -287,16 +288,16 @@ public class EcoreXMLSchemaBuilder extends MapBuilder
    * Each EEnum is mapped to a schema simple type that restricts the string simple type;
    * the schema type has an enumeration facet for each EEnumLiteral.
    */
-  protected void processEnum(EEnum enum)
+  protected void processEnum(EEnum eEnum)
   {
     XSDSimpleTypeDefinition enumType = XSDFactory.eINSTANCE.createXSDSimpleTypeDefinition();
-    enumType.setName(getName(enum));
+    enumType.setName(getName(eEnum));
     enumType.setBaseTypeDefinition(xsdSchema.getSchemaForSchema().resolveSimpleTypeDefinition("NCName"));
     xsdSchema.getContents().add(enumType);
 
-    map(enumType, enum);
+    map(enumType, eEnum);
 
-    for (Iterator literals = enum.getELiterals().iterator(); literals.hasNext();)
+    for (Iterator literals = eEnum.getELiterals().iterator(); literals.hasNext();)
     {
       EEnumLiteral literal = (EEnumLiteral)literals.next();
       XSDEnumerationFacet facet = XSDFactory.eINSTANCE.createXSDEnumerationFacet();
@@ -384,10 +385,13 @@ public class EcoreXMLSchemaBuilder extends MapBuilder
       List allSupers = ((EClass)superClasses.get(0)).getEAllSuperTypes();
       Set allSuperClasses = new HashSet();
       allSuperClasses.addAll(allSupers);
+      allSuperClasses.add(superClasses.get(0));
 
       for (int i = 1; i < superClasses.size(); i++)
       {
-        features.addAll(getAllFeatures((EClass)superClasses.get(i), allSuperClasses));
+        EClass superClass = (EClass)superClasses.get(i);
+        if (!allSuperClasses.contains(superClass))
+          features.addAll(getAllFeatures(superClass, allSuperClasses));
       }
     }
 
@@ -404,13 +408,16 @@ public class EcoreXMLSchemaBuilder extends MapBuilder
   {
     List features = new ArrayList();
     List superClasses = eClass.getESuperTypes();
-
+    
     for (int i = 0; i < superClasses.size(); i++)
       if (!classesToIgnore.contains(superClasses.get(i)))
         features.addAll(getAllFeatures((EClass)superClasses.get(i), classesToIgnore));
 
     features.addAll(eClass.getEAttributes());
     features.addAll(eClass.getEReferences());
+    
+    // the class has been processed and should be ignored if seen again in the hierarchy
+    classesToIgnore.add(eClass);
     return features;
   }
 
@@ -599,6 +606,10 @@ public class EcoreXMLSchemaBuilder extends MapBuilder
 
   protected void setUseToRequired(EAttribute attribute, XSDAttributeUse attrUse)
   {
+    if (attribute.isRequired())  
+    {
+      attrUse.setUse(XSDAttributeUseCategory.REQUIRED_LITERAL);
+    }
   }
 
   protected void setDefaultValue(EAttribute attribute, XSDAttributeDeclaration attrDecl)
