@@ -3,16 +3,16 @@
  *
  * Copyright (c) 2002-2004 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors: 
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: XSDComplexTypeDefinitionItemProvider.java,v 1.3 2004/04/15 05:43:24 marcelop Exp $
+ * $Id: XSDComplexTypeDefinitionItemProvider.java,v 1.5 2005/06/12 12:34:45 emerks Exp $
  */
 package org.eclipse.xsd.provider;
 
@@ -70,8 +70,6 @@ public class XSDComplexTypeDefinitionItemProvider
     {
       super.getPropertyDescriptors(object);
 
-      XSDComplexTypeDefinition xsdComplexTypeDefinition = ((XSDComplexTypeDefinition)object);
-
       // This is for the derivationMethod feature.
       //
       itemPropertyDescriptors.add
@@ -298,7 +296,6 @@ public class XSDComplexTypeDefinitionItemProvider
     if (childrenFeatures == null)
     {
       super.getChildrenFeatures(object);
-      XSDComplexTypeDefinition xsdComplexTypeDefinition = ((XSDComplexTypeDefinition)object);
       childrenFeatures.add(xsdPackage.getXSDTypeDefinition_Annotation());
       childrenFeatures.add(xsdPackage.getXSDComplexTypeDefinition_ContentAnnotation());
       childrenFeatures.add(xsdPackage.getXSDTypeDefinition_DerivationAnnotation());
@@ -311,7 +308,6 @@ public class XSDComplexTypeDefinitionItemProvider
 
   protected EStructuralFeature getChildFeature(Object object, Object child)
   {
-    XSDComplexTypeDefinition xsdComplexTypeDefinition = ((XSDComplexTypeDefinition)object);
     EObject refObject = (EObject)child;
     if (refObject instanceof XSDAttributeDeclaration)
     {
