@@ -3,16 +3,16 @@
  *
  * Copyright (c) 2002-2004 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors: 
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: XSDSimpleTypeDefinitionItemProvider.java,v 1.2 2004/04/07 22:12:58 davidms Exp $
+ * $Id: XSDSimpleTypeDefinitionItemProvider.java,v 1.4 2005/06/12 12:34:45 emerks Exp $
  */
 package org.eclipse.xsd.provider;
 
@@ -78,8 +78,6 @@ public class XSDSimpleTypeDefinitionItemProvider
     {
       super.getPropertyDescriptors(object);
 
-      XSDSimpleTypeDefinition xsdSimpleTypeDefinition = ((XSDSimpleTypeDefinition)object);
-
       // This is for the annotation feature.
       //
       itemPropertyDescriptors.add
@@ -333,7 +331,6 @@ public class XSDSimpleTypeDefinitionItemProvider
     if (childrenFeatures == null)
     {
       super.getChildrenFeatures(object);
-      XSDSimpleTypeDefinition xsdSimpleTypeDefinition = ((XSDSimpleTypeDefinition)object);
       childrenFeatures.add(xsdPackage.getXSDTypeDefinition_Annotation());
       childrenFeatures.add(xsdPackage.getXSDTypeDefinition_DerivationAnnotation());
       childrenFeatures.add(xsdPackage.getXSDSimpleTypeDefinition_Contents());
@@ -495,7 +492,6 @@ public class XSDSimpleTypeDefinitionItemProvider
         ((XSDComplexTypeDefinition) std.eContainer()).getDerivationMethod() != XSDDerivationMethod.EXTENSION_LITERAL)
     {
       boolean isAtomic = std.getVariety() == XSDVariety.ATOMIC_LITERAL;
-      boolean isList = std.getVariety() == XSDVariety.LIST_LITERAL;
       boolean isUnion = std.getVariety() == XSDVariety.UNION_LITERAL;
       Collection validFacets = std.getValidFacets();
 
